#include <stdio.h>

#ifdef USE_ALLEG
#include <allegro.h>
#endif

#include "globals.h"

/*	All assembler mnemonic commands without parameters.
*ADC *AND *ASL *BCC *BCS *BEQ *BIT *BMI *BNE *BPL *BRK *BVC *BVS *CLC
*CLD *CLI *CLV *CMP *CPX *CPY *DEC *DEX *DEY *EOR *INC *INX *INY *JMP
*JSR *LDA *LDX *LDY *LSR *NOP *ORA *PHA *PHP *PLA *PLP *ROL *ROR *RTI
*RTS !SBC *SEC *SED *SEI *STA *STX *STY *TAX *TAY *TSX *TXA *TXS *TYA
*/

__inline__ void j6502_adc ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	int tmp;
	_add_fun ( );
	tmp = prose.a + _exec_tmp + ( prose.flags[ CARRY ] ? 0 : 1 );

	prose.a = ( unsigned char )( tmp & 0xFF );

	prose.flags[ CARRY ] = ( tmp > 0xFF );
	prose.flags[ OVERFLOW ] = ( ~ ( prose.a ^ _exec_tmp ) & ( prose.a ^ tmp ) & 0x80 );

	set_ar_flags ( prose.a );
	
	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_and ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.a &= _exec_tmp;

	set_ar_flags ( prose.a );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_asl ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ CARRY ] = _exec_tmp > 0x7F;
	_exec_tmp <<= 1;
	set_ar_flags ( _exec_tmp );
//	ROM[ _exec_addr ] = ( unsigned char ) _exec_tmp;
	write_memory ( _exec_addr, _exec_tmp );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_bcc ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ CARRY ] == 0 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_bcs ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ CARRY ] == 1 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_beq ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ ZERO ] == 1 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_bit ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ SIGN ] = ( _exec_tmp > 0x7F );
	prose.flags[ OVERFLOW ] = ( ( _exec_tmp & 0x40 ) != 0 );
	prose.flags[ ZERO ] = ( prose.a == _exec_tmp );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_bmi ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ SIGN ] == 1 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_bne ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ ZERO ] == 0 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_bpl ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags [ SIGN ] == 0 )
		prose.ip += ( signed char ) _exec_tmp;

	add_cycle ( clocks );
}

__inline__ void j6502_brk ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip++;
	ROM[ 0x100 + prose.stack ] = (unsigned char ) ( prose.ip >> 8 );
	prose.stack--;
	ROM[ 0x100 + prose.stack ] = ( unsigned char ) ( prose.ip & 0xFF );
	prose.stack--;
	prose.flags[ BREAK ] = 1;
	ROM[ 0x100 + prose.stack ] = get_status ( );
	prose.flags[ INTERRUPT ] = 1;

	prose.ip = ( ROM[ 0xFFFF ] << 8 ) + ROM[ 0xFFFE ];
	add_cycle ( clocks );
}

__inline__ void j6502_bvc( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ OVERFLOW ] == 0 )
		prose.ip = _exec_addr;

	add_cycle ( clocks );
}

__inline__ void j6502_bvs( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip += cmd_tab[ code ].size;
	if ( prose.flags[ OVERFLOW ] == 1 )
		prose.ip = _exec_addr;

	add_cycle ( clocks );
}

__inline__ void j6502_clc ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ CARRY ] = 0;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_cld ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ DECIMAL ] = 0;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_cli ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ INTERRUPT ] = 0;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_clv ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ OVERFLOW ] = 0;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_cmp ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	_exec_tmp = prose.a - _exec_tmp;
/*	prose.flags[ CARRY ] = ( prose.a - _exec_tmp ) > 0x100;
	prose.flags[ SIGN ] = ( prose.a - _exec_tmp ) > 0x7F;
//	prose.flags[ ZERO ] = ( prose.a - _exec_tmp ) == 0;
	prose.flags[ ZERO ] = ( prose.a - _exec_tmp ) <= 0; */
	set_carry ( _exec_tmp < 0x100 );
	set_sign ( _exec_tmp );
	set_zero ( _exec_tmp &= 0xFF );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_cpx ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	_exec_tmp = prose.x - _exec_tmp;
/*	prose.flags[ CARRY ] = ( prose.x - _exec_tmp ) > 0x100;
	prose.flags[ SIGN ] = ( prose.x - _exec_tmp ) > 0x7F;
	prose.flags[ ZERO ] = ( prose.x - _exec_tmp ) == 0;
	prose.flags[ ZERO ] = ( prose.x == _exec_tmp ); */
	set_carry ( _exec_tmp < 0x100 );
	set_sign ( _exec_tmp );
	set_zero ( _exec_tmp &= 0xFF );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_cpy ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	_exec_tmp = prose.y - _exec_tmp;
/*	prose.flags[ CARRY ] = ( prose.y - _exec_tmp ) > 0x100;
	prose.flags[ SIGN ] = ( prose.y - _exec_tmp ) > 0x7F;
	prose.flags[ ZERO ] = ( prose.y - _exec_tmp ) == 0;
	prose.flags[ ZERO ] = ( prose.y ==_exec_tmp );*/
	set_carry ( _exec_tmp < 0x100 );
	set_sign ( _exec_tmp );
	set_zero ( _exec_tmp &= 0xFF );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_dec ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	_exec_tmp--;
//	ROM[ _exec_addr ] = _exec_tmp;
	write_memory ( _exec_addr, _exec_tmp );

//	prose.flags[ ZERO ] = ( _exec_tmp == 0 );
	prose.flags[ ZERO ] = ( _exec_tmp <= 0 );
	prose.flags[ SIGN ] = ( _exec_tmp > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_dex ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.x--;
//	prose.flags[ ZERO ] = ( prose.x == 0 );
	prose.flags[ ZERO ] = ( prose.x <= 0 );
	prose.flags[ SIGN ] = ( prose.x > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_dey ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.y--;
//	prose.flags[ ZERO ] = ( prose.y == 0 );
	prose.flags[ ZERO ] = ( prose.y <= 0 );
	prose.flags[ SIGN ] = ( prose.y > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_eor ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.a ^= _exec_tmp;
//	prose.flags[ ZERO ] = ( _exec_tmp == 0 );
	prose.flags[ ZERO ] = ( _exec_tmp <= 0 );
	prose.flags[ SIGN ] = ( _exec_tmp > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_inc ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	_exec_tmp++;
//	ROM[ _exec_addr ] = _exec_tmp;
	write_memory ( _exec_addr, _exec_tmp );

//	prose.flags[ ZERO ] = ( _exec_tmp == 0 );
	prose.flags[ ZERO ] = ( _exec_tmp <= 0 );
	prose.flags[ SIGN ] = ( _exec_tmp > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_inx ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.x++;
//	prose.flags[ ZERO ] = ( prose.x == 0 );
	prose.flags[ ZERO ] = ( prose.x <= 0 );
	prose.flags[ SIGN ] = ( prose.x > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_iny ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.y++;
//	prose.flags[ ZERO ] = ( prose.y == 0 );
	prose.flags[ ZERO ] = ( prose.y <= 0 );
	prose.flags[ SIGN ] = ( prose.y > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_jmp ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.ip = _exec_addr;

	add_cycle ( clocks );
}

__inline__ void j6502_jsr ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	ROM[ 0x100 + prose.stack ] = ( unsigned char ) ( prose.ip >> 8 );
	prose.stack--;
	ROM[ 0x100 + prose.stack ] = ( unsigned char ) ( prose.ip & 0xFF );
	prose.stack--;
	prose.ip = _exec_addr;

	add_cycle ( clocks );
}

__inline__ void j6502_lda ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	_exec_tmp = read_memory ( _exec_addr );		//		****!!!****
	prose.a = _exec_tmp;

//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_ldx ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	_exec_tmp = read_memory ( _exec_addr );		//		****!!!****
	prose.x = _exec_tmp;

//	prose.flags[ ZERO ] = ( prose.x == 0 );
	prose.flags[ ZERO ] = ( prose.x <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.x < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_ldy ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	exec_tmp = read_memory ( _exec_addr );		//		****!!!****
	prose.y = _exec_tmp;

//	prose.flags[ ZERO ] = ( prose.y == 0 );
	prose.flags[ ZERO ] = ( prose.y <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.y < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_lsr ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ CARRY ] = ( prose.a & 0x01 );
	prose.a >>= 1;
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_nop ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_ora ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.a |= _exec_tmp;
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_pha ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	ROM[ 0x100 + prose.stack ] = prose.a;
	prose.stack--;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_php ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	ROM[ 0x100 + prose.stack ] = get_status ( );
	prose.stack--;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_pla ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.stack++;
	prose.a = ROM[ 0x100 + prose.stack ];
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_plp ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.stack++;
	_exec_tmp = ROM[ 0x100 + prose.stack ];
	set_status ( _exec_tmp );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_rol ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	if ( prose.flags[ CARRY ] == 1 )
	{
		prose.flags[ CARRY ] = ( prose.a & 0x80 );
		prose.a = ( prose.a << 1 ) | 1;
	}
	else
	{
		prose.flags[ CARRY ] = ( prose.a & 0x80 );
		prose.a = ( prose.a << 1 );
	}
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( prose.a > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_ror ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	if ( prose.flags[ CARRY ] == 1 )
	{
		prose.flags[ CARRY ] = ( prose.a & 0x01 );
		prose.a = ( prose.a >> 1 ) | 0x80;
	}
	else
	{
		prose.flags[ CARRY ] = ( prose.a & 0x01 );
		prose.a = ( prose.a >> 1 );
	}
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( prose.a > 0x7F );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_rti ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.stack++;
	_exec_tmp = ROM[ 0x100 + prose.stack ];
	set_status ( _exec_tmp );
	prose.stack++;
	prose.ip = ( ROM[ 0x100 + prose.stack ] );
	prose.stack++;
	prose.ip += ( ROM[ 0x100 + prose.stack ] << 8 );		//	******* +3 ******

	add_cycle ( clocks );
}

__inline__ void j6502_rts ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.stack++;
	prose.ip = ( ROM[ 0x100 + prose.stack ] );
	prose.stack++;
	prose.ip += ( ROM[ 0x100 + prose.stack ] << 8 ) + 3;

	add_cycle ( clocks );
}

__inline__ void j6502_sbc ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	unsigned int tmp;
	_add_fun ( );
	tmp = prose.a - _exec_tmp;
	if ( prose.flags[ CARRY ] == 1 )
		tmp--;
	if ( tmp == 0xFF )
		prose.flags[ CARRY ] = ( prose.flags[ CARRY ] ? 1 : 0 );
	
	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_sec ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags [ CARRY ] = 1;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_sed ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags [ DECIMAL ] = 1;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_sei ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.flags[ INTERRUPT ] = 1;

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_sta ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	ROM[ _exec_addr ] = prose.a;
//	fprintf ( stderr, "ROM[ %X ] = %X; ( sta )\n", _exec_addr, prose.a );
	write_memory ( _exec_addr, prose.a );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_stx ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	ROM[ _exec_addr ] = prose.x;
//	fprintf ( stderr, "ROM[ %X ] = %X; ( stx )\n", _exec_addr, prose.x );
	write_memory ( _exec_addr, prose.x );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_sty ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
//	ROM[ _exec_addr ] = prose.y;
	write_memory ( _exec_addr, prose.y );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_tax ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.x = prose.a;
//	prose.flags[ ZERO ] = ( prose.x == 0 );
	prose.flags[ ZERO ] = ( prose.x <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.x < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_tay ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.y = prose.a;
//	prose.flags[ ZERO ] = ( prose.y == 0 );
	prose.flags[ ZERO ] = ( prose.y <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.y < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_tsx ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.x = prose.stack;
//	prose.flags[ ZERO ] = ( prose.x == 0 );
	prose.flags[ ZERO ] = ( prose.x <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.x < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_txa ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.a = prose.x;
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_txs ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.stack = prose.x;
//	prose.flags[ ZERO ] = ( prose.stack == 0 );
	prose.flags[ ZERO ] = ( prose.stack <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.stack < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_tya ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	_add_fun ( );
	prose.a = prose.y;
//	prose.flags[ ZERO ] = ( prose.a == 0 );
	prose.flags[ ZERO ] = ( prose.a <= 0 );
	prose.flags[ SIGN ] = ( ( signed char ) prose.a < 0 );

	prose.ip += cmd_tab[ code ].size;
	add_cycle ( clocks );
}

__inline__ void j6502_bad ( unsigned char code, unsigned char clocks,  void ( *_add_fun ) ( ) )
{
	printf ( "Bad opcode %x\nExit imminenet..\n", code );
	print_status ( );
	dump_rom ( );
	dump_ppu ( );
	exit ( 0 );
}
