/****************************
*							*
*	As global as it can be	*
*							*
****************************/

#define CARRY		0	//	C
#define ZERO		1	//	Z
#define INTERRUPT	2	//	I
#define DECIMAL 	3	//	D
#define BREAK		4	//	B
#define UNUSED		5	//	
#define OVERFLOW	6	//	V
#define SIGN		7	//	N

typedef struct
{
	unsigned char a;
	unsigned char x;
	unsigned char y;
	unsigned char flags[8];
	unsigned char stack;
	unsigned int ip;
	unsigned long clocks;
	unsigned char spr_addr;
} COREREGS;

typedef struct
{
	char asi[4];
} NAMESTRING;

typedef struct
{
	char prg_rom_count;
	char chr_rom_count;
	unsigned int mm_version;
} ROM_INFO;

typedef unsigned int word;
typedef unsigned char byte;
typedef unsigned long dword;

typedef struct
{
	char lexic[ 4 ];
	char size;
	char clocks;
	void ( *add_fun )( );
	void ( *handler )( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
} CMD;

/****************************
*							*
*		globals.c			*
*							*
****************************/

extern unsigned char ROM[0xFFFF];
extern unsigned char PPU[0xFFFF];
extern unsigned char SPR[0xFF];
extern FILE *nesfile;
extern unsigned char fileopen;
extern COREREGS prose;
extern ROM_INFO rom_info;

#ifdef USE_ALLEG
extern BITMAP* _v_screen;
extern PALETTE pal;
extern RGB nes_pal[64];
#endif

/****************************
*							*
*		newexec.c			*
*							*
****************************/

extern unsigned int _exec_tmp;
extern unsigned int _exec_addr;
extern unsigned int _read_tmp;
extern unsigned int _write_tmp;
extern const CMD cmd_tab[ ];


/****************************
*							*
*		newhelpers.c		*
*							*
****************************/

extern __inline__ void add_cycle ( unsigned char cycle );
extern __inline__ void page_check ( unsigned int addr1, unsigned int addr2 );
extern __inline__ void immediate ( );			//	Immediate				#$aa
extern __inline__ void absolute ( );			//	Absolute				$aaaa
extern __inline__ void abs_index_x ( );			//	Absolute Indexed X		$aaaa,X
extern __inline__ void abs_index_y ( );			//	Absolute Indexed Y		$aaaa,Y
extern __inline__ void zeropage ( );			//	Zero page				$00aa
extern __inline__ void zp_index_x ( );			//	Zero Page Indexed X		$00aa,X
extern __inline__ void zp_index_y ( );			//	Zero Page Indexed Y		$00aa,Y
extern __inline__ void indexed_x ( );			//	Indexed Indirect 		($aa),X
extern __inline__ void indexed_y ( );			//	Indirect Indexed		($aa),Y
extern __inline__ void jmp_indirect ( );		//	** ONLY USED ON JMP COMMAND **
extern __inline__ void relative ( );			//	Branch asjad
extern __inline__ void implied ( );				//	Implied					(no arg)

extern __inline__ void set_carry ( unsigned int status );
extern __inline__ void set_overflow ( unsigned int status );
extern __inline__ void set_interrupt ( unsigned int status );
extern __inline__ void set_break ( unsigned int status );
extern __inline__ void set_decimal ( unsigned int status );
extern __inline__ void set_zero ( unsigned int status );
extern __inline__ void set_sign ( unsigned int status );

extern __inline void set_ar_flags ( unsigned char value );	//	Sets arithmetic flags
extern __inline__ unsigned char get_status ( );				// Makes unsigned char from flags
extern __inline__ void set_status ( unsigned char status );	// Sets flags based on unsigned char
extern void get_rom_info ( );
extern void reset_cpu ( );
extern int read_prg ( int page, unsigned int address, unsigned int size );
extern int read_chr ( int page, unsigned int address, unsigned int size );

extern __inline__ void write_memory(unsigned int _write_addr, unsigned char _write_val );
extern __inline__ unsigned char read_memory ( unsigned int _read_addr );

/****************************
*							*
*			cmds.c			*
*							*
****************************/

extern __inline__ void j6502_adc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_and ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_asl ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bcc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bcs ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_beq ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bit ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bmi ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bne ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bpl ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_brk ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bvc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bvs ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_clc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_cld ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_cli ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_clv ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_cmp ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_cpx ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_cpy ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_dec ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_dex ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_dey ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_eor ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_inc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_inx ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_iny ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_jmp ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_jsr ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_lda ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_ldx ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_ldy ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_lsr ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_nop ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_ora ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_pha ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_php ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_pla ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_plp ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_rol ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_ror ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_rti ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_rts ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sbc ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sec ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sed ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sei ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sta ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_stx ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_sty ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_tax ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_tay ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_tsx ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_txa ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_txs ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_tya ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );
extern __inline__ void j6502_bad ( unsigned char code, unsigned char clocks, void ( *_add_fun ) ( ) );


/****************************
*							*
*		mappers.c			*
*							*
****************************/

/****************************
*							*
*			ppu.c			*
*							*
****************************/

extern unsigned int ppuAddressOld;
extern unsigned int ppuAddressNew;
extern unsigned int ppuControl1;
extern unsigned int ppuControl2;
extern unsigned int ppuStatus;
extern unsigned int ppuAddressHigh;

extern void exec_ppu ( );
extern void dump_ppu ( );
extern __inline__ void  write_ppu ( unsigned int _ppu_addr, unsigned char _ppu_val );
extern void do_bg_scanline ( unsigned int _sc_nr );
extern void init_my_nes_pal ( );
extern void test_sprite_func ( );

/****************************
*							*
*			main.c			*
*							*
****************************/

extern void print_smth ( );
extern void print_status ( );
extern void print_status_lite ( );

/****************************
*							*
*		interrupt.c			*
*							*
****************************/
extern void NMI ( );
