#include <stdio.h>

#ifdef USE_ALLEG
#include <allegro.h>
#endif

#include "globals.h"

/*
**	int	_exec_tmp	hoiab ajutist vrtust, mis on loetud antud ROM asukohast
**	int	_exec_addr	hoiad ajutist mlu aadressi, kust on miskit vaja lugeda
*/

/*
**	It appears that I have to make read_memory and write_memory functions,
**	in order to get PPU done..
**	Part of code is idea from damadman @ #fakenes irc.openprojects.net
*/

unsigned int _read_tmp;
unsigned int _write_tmp;

__inline__ unsigned char read_memory ( unsigned int _read_addr )
{
	if ( _read_addr < 0x2000 )
		return ( ROM[ _read_addr ] );
	if ( _read_addr == 0x2002 )
		return ppuStatus;
	if ( _read_addr == 0x2007 )
	{
		_read_tmp = ppuAddressOld;
		ppuAddressOld = ppuAddressNew;
		if ( ( ppuControl1 & 0x04 ) == 0 )
			ppuAddressNew++;
		else
			ppuAddressNew += 0x20;
		return PPU[ _read_tmp ];
	}
	return ROM[ _read_addr ];
}

__inline__ void write_memory(unsigned int _write_addr, unsigned char _write_val )
{
	if( ( _write_addr > 0x1FFF ) & ( _write_addr < 0x4000 ) )
	{
		write_ppu( _write_addr, _write_val );
		return;
	}
	if ( _write_addr == 0x4014 )
	{
//		printf ( "SPR DMA:\n" );
//		print_status ( );
		write_ppu ( _write_addr, _write_val );
		return;
	}
	ROM[ _write_addr ] = _write_val;
}

//		Kella seadmise stuff

__inline__ void add_cycle ( unsigned char cycle )
{
	prose.clocks += cycle;
}

__inline__ void page_check ( unsigned int addr1, unsigned int addr2 )
{
	prose.clocks += ( ( ( addr1 >> 8 ) == ( ( addr1 + addr2 ) >> 8 ) ) ? 0 : 1 );
}

//		Adresseerimise viisid

__inline__ void implied ( )				//				-----
{
}

__inline__ void immediate ( )			//				$aa
{
	_exec_addr = prose.ip + 1;
	_exec_tmp = ROM[ _exec_addr ];
}

__inline__ void absolute ( )			//				$aaaa
{
	_exec_addr = ( ( ROM[ prose.ip + 2 ] & 0xFF ) << 8 ) + ( ROM[ prose.ip + 1 ] & 0xFF );
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
}

__inline__ void zeropage ( )			//				$00aa
{
	_exec_addr = ROM[ prose.ip + 1 ];
	_exec_tmp = ROM[ _exec_addr ];
}

__inline__ void jmp_indirect ( )		// ** ONLY USED ON JMP COMMAND **
{
	absolute ( );
	if ( 0xFF == ( _exec_tmp & 0xFF ) )
		_exec_addr = ( ROM[ _exec_tmp & 0xFF00 ] << 8 ) + ROM[ _exec_tmp ];
	else
		_exec_addr = ROM[ _exec_tmp ];
}

__inline__ void abs_index_x ( )		// Absolute Indexed		$aaaa,X
{
	absolute ( );
	_exec_addr = ( _exec_addr + prose.x ) & 0xFFFF;
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
	page_check ( _exec_addr, prose.x );
}

__inline__ void abs_index_y ( )		// Absolute Indexed		$aaaa,Y
{
	absolute ( );
	_exec_addr = ( _exec_addr + prose.y ) & 0xFFFF;
	_exec_tmp = read_memory( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
	page_check ( _exec_addr, prose.y );
}

__inline__ void zp_index_x ( )			// Zero Page Indexed	$00aa,X
{
	_exec_addr = ( ROM[ prose.ip + 1 ] + prose.x ) & 0xFFFF;
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
	page_check ( _exec_addr, prose.x );
}

__inline__ void zp_index_y ( )			// Zero Page Indexed	$00aa,Y
{
	_exec_addr = ( ROM[ prose.ip + 1 ] + prose.y ) & 0xFFFF;
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
	page_check ( _exec_addr, prose.y );
}

__inline__ void indexed_x ( )			// Indexed Indirect ($aa),X
{
	_exec_addr = ROM[ prose.ip + 1 ];
	_exec_addr = ( ( ROM[ _exec_addr + 1 ] & 0xFF ) << 8 ) + ( ROM[ _exec_addr ] & 0xFF );
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
}

__inline__ void indexed_y ( )			// Indirect Indexed	($aa),Y
{
	_exec_addr = ROM[ prose.ip + 1 ];
	_exec_addr = ( ( ROM[ _exec_addr + 1 ] & 0xFF ) << 8 ) + ( ROM[ _exec_addr ] & 0xFF );
	_exec_addr += prose.y;
	_exec_tmp = read_memory ( _exec_addr );
//	_exec_tmp = ROM[ _exec_addr ];
	page_check ( _exec_addr, prose.y );
}

__inline__ void relative ( )			// Branch asjad
{
	_exec_tmp = ROM[ prose.ip + 1 ];
	if ( ( prose.ip >> 8 ) == ( _exec_tmp >> 8 ) )
		add_cycle ( 1 );
	else
		add_cycle ( 2 );
	_exec_addr = prose.ip + ROM[ prose.ip + 1 ];
}

__inline__ void set_ar_flags ( unsigned char value )
{
	prose.flags[ ZERO ] = ( value == 0 );
	prose.flags[ SIGN ] = ( ( signed char ) value < 0 );
}

__inline__ unsigned char get_status ( )
{
	return ( ( prose.flags[ SIGN ] & 0x80 ) | ( prose.flags[ OVERFLOW ] ? 0x40 : 0 ) |
		( prose.flags[ UNUSED ] ? 0x20 : 0x20 ) | ( prose.flags[ BREAK ] ? 10 : 0 ) |
		( prose.flags[ DECIMAL ] ? 0x08 : 0 ) | ( prose.flags[ INTERRUPT ] ? 0x04 : 0 ) |
		( prose.flags[ ZERO ] ? 0x02 : 0 ) | ( prose.flags[ CARRY ] ? 0x01 : 0 ) );
}

__inline__ void set_status ( unsigned char status )
{
	prose.flags[ SIGN ] = ( status & 0x80 );
	prose.flags[ OVERFLOW ] = ( status & 0x40 );
	prose.flags[ BREAK ] = ( status & 0x10 );
	prose.flags[ DECIMAL ] = ( status & 0x08 );
	prose.flags[ INTERRUPT ] = ( status & 0x04 );
	prose.flags[ ZERO ] = ( status & 0x02 );	//	********!!!!!!********
	prose.flags[ CARRY ] = ( status & 0x01 );
}

__inline__ void set_carry ( unsigned int status )
{
	if ( status != 0 )
		prose.flags[ CARRY ] = 1;
	else
		prose.flags[ CARRY ] = !( prose.flags[ CARRY ] );
}

__inline__ void set_overflow ( unsigned int status )
{
	if ( status != 0 )
		prose.flags[ OVERFLOW ] = 1;
	else
		prose.flags[ OVERFLOW ] = !( prose.flags[ OVERFLOW ] );
}

__inline__ void set_interrupt ( unsigned int status )
{
	if ( status != 0 )
		prose.flags[ INTERRUPT ] = 1;
	else
		prose.flags[ INTERRUPT ] = !( prose.flags[ INTERRUPT ] );
}

__inline__ void set_break ( unsigned int status )
{
	if ( status != 0 )
		prose.flags[ BREAK ] = 1;
	else
		prose.flags[ BREAK ] = !( prose.flags[ BREAK ] );
}

__inline__ void set_decimal ( unsigned int status )
{
	if ( status != 0 )
		prose.flags[ DECIMAL ] = 1;
	else
		prose.flags[ DECIMAL ] = !( prose.flags[ DECIMAL ] );
}

__inline__ void set_zero ( unsigned int status )
{
	if ( status == 0 )
		prose.flags[ ZERO ] = 1;
	else
		prose.flags[ ZERO ] = 0;
}

__inline__ void set_sign ( unsigned int status )
{
	if ( status == 0 )
		prose.flags[ SIGN ] = 1;
	else
		prose.flags[ SIGN ] = 0;
}

void get_rom_info ( )
{
	int a;
	if ( fileopen == 0 )
	{
		printf ( "File not opened!\n" );
		exit ( 0 );
	}
	for ( a = 0; a < 4; a++ )
		getc ( nesfile );
	rom_info.prg_rom_count = getc ( nesfile );
	rom_info.chr_rom_count = getc ( nesfile );
	rom_info.mm_version = getc ( nesfile );
	rom_info.mm_version = rom_info.mm_version >> 4;
	rom_info.mm_version += getc ( nesfile );
}

void reset_cpu ( )
{
	int a;
	prose.a = prose.x = prose.y = 0;
	prose.stack = 0xff;
	prose.clocks = ( long ) 0;
	prose.ip = ( ROM[0xFFFD] << 8 ) | ROM[0xFFFC];
	for ( a = 0; a < 8; a++ )
		prose.flags[a] = 0;
}

int read_prg ( int page, unsigned int address, unsigned int size )
{
	if ( fileopen == 0 )
	{
		printf ( "ERROROROROR!\n" );
		return -1;
	}
	fseek ( nesfile, ( page * 16384 + 16 ), SEEK_SET );
	fread ( &ROM[address], size, 1, nesfile );
	return 0;
}

int read_chr ( int page, unsigned int address, unsigned int size )
{
	if ( fileopen == 0 )
	{
		printf ( "ERROROROROR!\n" );
		return -1;
	}
	fseek ( nesfile, ( rom_info.prg_rom_count * 0x4000 + page * 2000 + 16 ), SEEK_SET );
	fread ( &PPU[address], size, 1, nesfile );
	return 0;
}
